/**
 *
 */
package edu.cmu.radar.dynaweb.actions;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import edu.cmu.radar.dynaweb.entities.Location;
import edu.cmu.radar.dynaweb.entities.LocationCapacity;
import edu.cmu.radar.dynaweb.utils.StringUtils;

/**
 * @author Jialiang
 *
 * The base action that provides room operations
 */
public class RoomAction extends BaseAction {
	/**
	 * gets the room-capacity names
	 */
	protected List<String> getCapacityNames(List<Location> roomList) {
		List capNameList = new LinkedList();
		if (roomList != null && !roomList.isEmpty()) {
			// get the first room and put its capacity names into a list
			Location sampleLocation = roomList.get(0);
			Set<LocationCapacity> locSet = sampleLocation.getCapacities();
			for (LocationCapacity cap : locSet) {
				String name = cap.getConfiguration();
				capNameList.add(name);
			}
		}

		return capNameList;
	}

	/**
	 * capitalizes the first character of the capacity names
	 */
	protected List<String> getFormattedCapacityNames(
			List<String> capacityNameList) {
		List<String> formattedCapNameList = new LinkedList<String>();
		for (String capName : capacityNameList) {
			formattedCapNameList.add(StringUtils.toTitleCase(capName));
		}

		return formattedCapNameList;
	}

	/**
	 * loads rooms and their capacities into maps, and formats the capacity
	 * names
	 */
	protected void loadRooms(String building, String type,
			Map<String, List<Room>> roomsMap,
			Map<String, List<String>> fmtCapNamesMap, CapacityFormat format) {
		List<Location> locations = services.getLocation(building, type);
		List<String> capNames = this.getCapacityNames(locations);
		List<Room> rooms = getRooms(locations, capNames, format);
		roomsMap.put(type, rooms);

		List<String> fmtCapNames = this.getFormattedCapacityNames(capNames);
		fmtCapNamesMap.put(type, fmtCapNames);
	}

	/**
	 * parses the capacity set into a sorted list in the order of capacity names
	 */
	protected List<Room> getRooms(List<Location> locations,
			List<String> capNames, CapacityFormat format) {
		List<Room> rooms = new LinkedList<Room>();

		for (Location loc : locations) {
			Room locBean = new Room(loc.getCaption());

			List capacities = locBean.getCapacities();
			// load the capacity value in the order of capacity names
			for (String cap : capNames) {
				Set<LocationCapacity> capacitySet = loc.getCapacities();
				for (LocationCapacity locCap : capacitySet) {
					if (cap.equals(locCap.getConfiguration())) {
						if (format != null) {
							String fmtCapVal = format.format(cap, locCap
									.getCapacity());
							LocationCapacity fmtCap = new LocationCapacity(loc,
									cap, fmtCapVal);
							capacities.add(fmtCap);
						} else {
							capacities.add(locCap);
						}
					}
				}
			}
			rooms.add(locBean);
		}

		return rooms;
	}

	/**
	 * delegate method for {@link #getRooms(List, List, CapacityFormat)}
	 */
	protected List<Room> getRooms(List<Location> locations,
			List<String> capNames) {
		return getRooms(locations, capNames, null);
	}

	/**
	 * delegate method {@link #loadRooms(String, String, Map, Map, CapacityFormat)}
	 */
	protected void loadRooms(String building, String type,
			Map<String, List<Room>> roomsMap,
			Map<String, List<String>> fmtCapNamesMap) {
		loadRooms(building, type, roomsMap, fmtCapNamesMap, null);
	}
}
