/**
 *
 */
package edu.cmu.radar.dynaweb.actions;

import java.text.DecimalFormat;
import java.text.Format;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * @author Jialiang
 *
 * A struts action that provides room data for Stever Hall (SHrooms.jsp)
 */
public class SteverRoomSpecAction extends RoomAction {
	/**
	 * the room list categorized by types
	 */
	private Map<String, List<Room>> roomsMap = new HashMap<String, List<Room>>();

	/**
	 * the list of formatted capacity names categorized by types
	 */
	private Map<String, List<String>> fmtCapNamesMap = new HashMap<String, List<String>>();

	/**
	 * the room capacity formatter
	 */
	private CapacityFormat format = new SteverSpecCapacityFormat();

	/*
	 * (non-Javadoc)
	 *
	 * @see com.opensymphony.xwork2.ActionSupport#execute()
	 */
	public String execute() throws Exception {
		String building = "Stever";
		loadRooms(building, "classroom", roomsMap, fmtCapNamesMap, format);
		loadRooms(building, "conference", roomsMap, fmtCapNamesMap, format);
		loadRooms(building, "auditorium", roomsMap, fmtCapNamesMap, format);
		loadRooms(building, "reception", roomsMap, fmtCapNamesMap, format);

		return SUCCESS;
	}

	/**
	 * @return the roomsMap
	 */
	public Map<String, List<Room>> getRoomsMap() {
		return roomsMap;
	}

	/**
	 * @return the fmtCapNamesMap
	 */
	public Map<String, List<String>> getFmtCapNamesMap() {
		return fmtCapNamesMap;
	}

}

/**
 * A room capacity formatter that formats decimal values in the pattern of
 * "#,###", and concerts the dimension into inches and feet
 */
class SteverSpecCapacityFormat implements CapacityFormat {
	private Format squareFormat = new DecimalFormat("#,###");

	private Pattern dimensionPattern = Pattern
			.compile("\\d*\\.\\d+'\\s*[Xx]\\s*\\d*.\\d+'");

	/*
	 * (non-Javadoc)
	 *
	 * @see edu.cmu.radar.dynaweb.actions.CapacityFormat#format(java.lang.String,
	 *      java.lang.String)
	 */
	public String format(String configuration, String capacity) {
		if (configuration == null || capacity == null) {
			return "n/a";
		}
		try {
		if (!"0".equals(capacity) && (Integer.parseInt(capacity)==0)) {
			return "n/a";
		}
		} catch (Exception e) {
			return "n/a";
		}

		try {
			if ("square".equalsIgnoreCase(configuration)) {
				int square = Integer.parseInt(capacity);
				return squareFormat.format(square);
			} else if ("dimension".equalsIgnoreCase(configuration)) {
				if (dimensionPattern.matcher(capacity).matches()) {
					return formatFeet(capacity);
				}
			}

			return capacity;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	private String formatFeet(String feet) {
		String[] segs = feet.split("\\s*[Xx]\\s*");
		// remove the feet mark
		String left = segs[0];
		String right = segs[1];
		left = left.substring(0, left.length() - 1);
		right = right.substring(0, right.length() - 1);

		double num1 = Double.parseDouble(left);
		double num2 = Double.parseDouble(right);

		int feet1 = (int) num1;
		int feet2 = (int) num2;

		int inch1 = (int) ((num1 - feet1) * 12);
		int inch2 = (int) ((num2 - feet2) * 12);

		StringBuilder buf = new StringBuilder(20);
		buf.append(feet1).append("'").append(inch1).append("\"");
		buf.append("&nbsp;x&nbsp;");
		buf.append(feet2).append("'").append(inch2).append("\"");

		return buf.toString();
	}

}
